package com.ims.studentManager;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.ims.studentManager.service.StudentQueryService;
import com.ims.studentManager.service.StudentService;
import com.ims.studentManger.Package.Student;

@Component
public class StudentManagerRunApp {
	
	@Autowired
	StudentService studentService;
	
	@Autowired
	StudentQueryService studentQueryService;
	
	StudentManagerHelp help;
	
	public void run(String args[]) {
		
		if(args.length < 1) {
			System.out.println("Error de sintaxis");
			//System.out.println(help.getHelp());
		} else if(args.length == 1) {
			switch(args[0].toLowerCase()) {
				case "-lm":
					studentService.findAllMarks().forEach(System.out::println);
					break;
				case "-h":
					System.out.println(help.getHelp());
					break;
				default:
					System.out.println("Error de sintaxis");
					//System.out.println(help.getHelp());
			}
			
		} else if (args.length % 2 != 0) {
			System.out.println("Error de sintaxis");
			//System.out.println(help.getHelp());
		} else if (args.length > 8) {
			System.out.println("Error de sintaxis");
			//System.out.println(help.getHelp());
		} else {
			List<String[]> argumentos = new ArrayList<>();
			
			for(int i = 0;i<args.length;i+=2) {
				argumentos.add(new String[] {args[i], args[i+1]});
			}
			
			boolean error = false;
			
			for(String[] argumento : argumentos) {
				switch(argumento[0].toLowerCase()) {
					case "-am":
						studentQueryService.anyMark(argumento[1].split(","));
						break;
					case "-tg":
						studentQueryService.allMarks(argumento[1].split(","));
						break;
					case "-g":
						studentQueryService.grade(argumento[1]);
						break;
					case "-b":
						String[] marks = argumento[1].split(",");
						studentQueryService.betweenAvgMark(Float.parseFloat(marks[0]), Float.parseFloat(marks[1]));
						break;
					case "-t":
						studentQueryService.nameContains(argumento[1]);
						break;
					default: error = true;
						System.out.println("Error de sintaxis");
						//System.out.println(help.getHelp());
				}
			}
			
			if(!error) {
				System.out.println("Hola");
				Collection<Student> result = studentQueryService.exec();
				System.out.printf("%s\t%-50s\t%s\t%s\n","ID","NAME","GRADE","MARKS");
				if(result != null) {
					result.forEach(f -> System.out.printf("%s\t%-50s\t%s\t%s\n",f.getId()
										,f.getName(),f.getGrade(),
										f.getMarks().stream().collect(Collectors.joining(", "))));
				}else {
					System.out.println("No hay Alumnos con esos criterios. No lo siento, deja de usarme");
				}
			}
			
		}
		
	}
	
}
