package com.ims.studentManager.service;

import java.util.Arrays;
import java.util.Collection;
import java.util.function.Predicate;
import java.util.stream.Collectors;

import javax.annotation.PostConstruct;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.ims.studentManager.dao.StudentDao;
import com.ims.studentManger.Package.Student;

@Service
public class StudentQueryServiceImpl implements StudentQueryService {
	
	@Autowired
	private StudentDao dao;
	
	private Predicate<Student> predicate;
	
	@PostConstruct
	public void init() {
		predicate = null;
	}
	
	@Override
	public Collection<Student> exec() {
		// @formatter:off
		return dao.findAll()
				.stream()
				.filter(predicate)
				.collect(Collectors.toList());
				
		// @formatter:on

	}

	@Override
	public StudentQueryService anyMark(String... marks) {
		Predicate<Student> pAnyMark = (student -> Arrays.stream(marks).allMatch(student.getMarks()::contains)); 
		predicate = (predicate == null) ? pAnyMark : predicate.and(pAnyMark);
		return this;
	}

	@Override
	public StudentQueryService allMarks(String... marks) {
		Predicate<Student> pAllMarks = (student -> Arrays.stream(marks).anyMatch(student.getMarks()::contains));
		predicate = (predicate == null) ? pAllMarks : predicate.and(pAllMarks);
		return this;
	}

	@Override
	public StudentQueryService grade(String grade) {
		Predicate<Student> pGrade = (student -> student.getGrade().equalsIgnoreCase(grade));
		predicate = (predicate == null) ? pGrade : predicate.and(pGrade);
		return this;
	}

	@Override
	public StudentQueryService betweenAvgMark(float max, float min) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public StudentQueryService nameContains(String name) {
		Predicate<Student> pNameContains = (student -> student.getName().toLowerCase().contains(name.toLowerCase()));
		predicate = (predicate == null) ? pNameContains : predicate.and(pNameContains);
		return this;
	}

}
